@@ -6,42 +6,41 @@
 //
 
 #define SOCI_EMPTY_SOURCE
-#include "soci.h"
-#include "soci-empty.h"
+#include "soci/empty/soci-empty.h"
 
 #ifdef _MSC_VER
 #pragma warning(disable:4355)
 #endif
 
-using namespace SOCI;
-using namespace SOCI::details;
+using namespace soci;
+using namespace soci::details;
 
 
-void EmptyVectorUseTypeBackEnd::bindByPos(int & /* position */,
-        void * /* data */, eExchangeType /* type */)
+void empty_vector_use_type_backend::bind_by_pos(int & /* position */,
+        void * /* data */, exchange_type /* type */)
 {
     // ...
 }
 
-void EmptyVectorUseTypeBackEnd::bindByName(
+void empty_vector_use_type_backend::bind_by_name(
     std::string const & /* name */, void * /* data */,
-    eExchangeType /* type */)
+    exchange_type /* type */)
 {
     // ...
 }
 
-void EmptyVectorUseTypeBackEnd::preUse(eIndicator const * /* ind */)
+void empty_vector_use_type_backend::pre_use(indicator const * /* ind */)
 {
     // ...
 }
 
-std::size_t EmptyVectorUseTypeBackEnd::size()
+std::size_t empty_vector_use_type_backend::size()
 {
     // ...
     return 1;
 }
 
-void EmptyVectorUseTypeBackEnd::cleanUp()
+void empty_vector_use_type_backend::clean_up()
 {
     // ...
 }
