@@ -85,6 +85,10 @@ void firebird_standard_into_type_backend::exchangeData()
         case x_long_long:
             exchange_type_cast<x_long_long>(data_) = from_isc<long long>(var);
             break;
+        case x_unsigned_long_long:
+            exchange_type_cast<x_unsigned_long_long>(data_) =
+                from_isc<unsigned long long>(var);
+            break;
         case x_double:
             exchange_type_cast<x_double>(data_) = from_isc<double>(var);
             break;
@@ -128,11 +132,17 @@ void firebird_standard_into_type_backend::exchangeData()
             break;
 
         case x_longstring:
-            copy_from_blob(exchange_type_cast<x_longstring>(data_).value);
+            {
+                std::string &tmp = exchange_type_cast<x_longstring>(data_).value;
+                copy_from_blob(statement_, buf_, tmp);
+            }
             break;
 
         case x_xmltype:
-            copy_from_blob(exchange_type_cast<x_xmltype>(data_).value);
+            {
+                std::string &tmp = exchange_type_cast<x_xmltype>(data_).value;
+                copy_from_blob(statement_, buf_, tmp);
+            }
             break;
 
         default:
@@ -140,29 +150,6 @@ void firebird_standard_into_type_backend::exchangeData()
     } // switch
 }
 
-void firebird_standard_into_type_backend::copy_from_blob(std::string& out)
-{
-    firebird_blob_backend blob(statement_.session_);
-
-    GCC_WARNING_SUPPRESS(cast-align)
-
-    blob.assign(*reinterpret_cast<ISC_QUAD*>(buf_));
-
-    GCC_WARNING_RESTORE(cast-align)
-
-    std::size_t const len_total = blob.get_len();
-    out.resize(len_total);
-
-    std::size_t const len_read = blob.read(0, &out[0], len_total);
-    if (len_read != len_total)
-    {
-        std::ostringstream os;
-        os << "Read " << len_read << " bytes instead of expected "
-           << len_total << " from Firebird text blob object";
-        throw soci_error(os.str());
-    }
-}
-
 void firebird_standard_into_type_backend::clean_up()
 {
     if (buf_ != NULL)
