@@ -116,6 +116,9 @@ void firebird_standard_use_type_backend::exchangeData()
         case x_long_long:
             to_isc<long long>(data_, var);
             break;
+        case x_unsigned_long_long:
+            to_isc<unsigned long long>(data_, var);
+            break;
         case x_double:
             to_isc<double>(data_, var);
             break;
@@ -163,7 +166,10 @@ void firebird_standard_use_type_backend::exchangeData()
 
 void firebird_standard_use_type_backend::copy_to_blob(const std::string& in)
 {
-    blob_ = new firebird_blob_backend(statement_.session_);
+    if (blob_)
+        blob_->trim(0);
+    else
+        blob_ = new firebird_blob_backend(statement_.session_);
     blob_->append(in.c_str(), in.length());
     blob_->save();
     memcpy(buf_, &blob_->bid_, sizeof(blob_->bid_));
