@@ -6,26 +6,27 @@
 //
 
 #define SOCI_FIREBIRD_SOURCE
-#include "soci-firebird.h"
-#include "error.h"
+#include "soci/firebird/soci-firebird.h"
+#include "firebird/error-firebird.h"
 #include <cctype>
 #include <sstream>
+#include <iostream>
 
-using namespace SOCI;
-using namespace SOCI::details;
-using namespace SOCI::details::Firebird;
+using namespace soci;
+using namespace soci::details;
+using namespace soci::details::firebird;
 
-FirebirdStatementBackEnd::FirebirdStatementBackEnd(FirebirdSessionBackEnd &session)
-        : session_(session), stmtp_(0), sqldap_(NULL), sqlda2p_(NULL), rowsFetched_(0),
-        intoType_(eStandard), useType_(eStandard), procedure_(false), boundByName_(false),
-		boundByPos_(false)
+firebird_statement_backend::firebird_statement_backend(firebird_session_backend &session)
+    : session_(session), stmtp_(0), sqldap_(NULL), sqlda2p_(NULL),
+        boundByName_(false), boundByPos_(false), rowsFetched_(0), endOfRowSet_(false), rowsAffectedBulk_(-1LL),
+            intoType_(eStandard), useType_(eStandard), procedure_(false)
 {}
 
-void FirebirdStatementBackEnd::prepareSQLDA(XSQLDA ** sqldap, int size)
+void firebird_statement_backend::prepareSQLDA(XSQLDA ** sqldap, short size)
 {
     if (*sqldap != NULL)
     {
-        *sqldap = reinterpret_cast<XSQLDA*>(realloc(*sqldap,XSQLDA_LENGTH(size)));
+        *sqldap = reinterpret_cast<XSQLDA*>(realloc(*sqldap, XSQLDA_LENGTH(size)));
     }
     else
     {
@@ -36,27 +37,29 @@ void FirebirdStatementBackEnd::prepareSQLDA(XSQLDA ** sqldap, int size)
     (*sqldap)->version = 1;
 }
 
-void FirebirdStatementBackEnd::alloc()
+void firebird_statement_backend::alloc()
 {
     ISC_STATUS stat[stat_size];
 
     if (isc_dsql_allocate_statement(stat, &session_.dbhp_, &stmtp_))
     {
-        throwISCError(stat);
+        throw_iscerror(stat);
     }
 }
 
-void FirebirdStatementBackEnd::cleanUp()
+void firebird_statement_backend::clean_up()
 {
+    rowsAffectedBulk_ = -1LL;
+
     ISC_STATUS stat[stat_size];
 
-    if (stmtp_ != NULL)
+    if (stmtp_ != 0)
     {
         if (isc_dsql_free_statement(stat, &stmtp_, DSQL_drop))
         {
-            throwISCError(stat);
+            throw_iscerror(stat);
         }
-        stmtp_ = NULL;
+        stmtp_ = 0;
     }
 
     if (sqldap_ != NULL)
@@ -72,7 +75,7 @@ void FirebirdStatementBackEnd::cleanUp()
     }
 }
 
-void FirebirdStatementBackEnd::rewriteParameters(
+void firebird_statement_backend::rewriteParameters(
     std::string const & src, std::vector<char> & dst)
 {
     std::vector<char>::iterator dst_it = dst.begin();
@@ -86,50 +89,50 @@ void FirebirdStatementBackEnd::rewriteParameters(
     int position = 0;
 
     for (std::string::const_iterator it = src.begin(), end = src.end();
-            it != end; ++it)
+        it != end; ++it)
     {
         switch (state)
         {
-            case eNormal:
-                if (*it == '\'')
-                {
-                    *dst_it++ = *it;
-                    state = eInQuotes;
-                }
-                else if (*it == ':')
-                {
-                    state = eInName;
-                }
-                else // regular character, stay in the same state
-                {
-                    *dst_it++ = *it;
-                }
-                break;
-            case eInQuotes:
-                if (*it == '\'')
-                {
-                    *dst_it++ = *it;
-                    state = eNormal;
-                }
-                else // regular quoted character
-                {
-                    *dst_it++ = *it;
-                }
-                break;
-            case eInName:
-                if (std::isalnum(*it) || *it == '_')
-                {
-                    name += *it;
-                }
-                else // end of name
-                {
-                    names_.insert(std::pair<std::string, int>(name, position++));
-                    name.clear();
-                    *dst_it++ = '?';
-                    *dst_it++ = *it;
-                    state = eNormal;
-                }
-                break;
+        case eNormal:
+            if (*it == '\'')
+            {
+                *dst_it++ = *it;
+                state = eInQuotes;
+            }
+            else if (*it == ':')
+            {
+                state = eInName;
+            }
+            else // regular character, stay in the same state
+            {
+                *dst_it++ = *it;
+            }
+            break;
+        case eInQuotes:
+            if (*it == '\'')
+            {
+                *dst_it++ = *it;
+                state = eNormal;
+            }
+            else // regular quoted character
+            {
+                *dst_it++ = *it;
+            }
+            break;
+        case eInName:
+            if (std::isalnum(*it) || *it == '_')
+            {
+                name += *it;
+            }
+            else // end of name
+            {
+                names_.insert(std::pair<std::string, int>(name, position++));
+                name.clear();
+                *dst_it++ = '?';
+                *dst_it++ = *it;
+                state = eNormal;
+            }
+            break;
         }
     }
 
@@ -146,7 +149,7 @@ namespace
 {
     int statementType(isc_stmt_handle stmt)
     {
-        int statement_type;
+        int stype;
         int length;
         char type_item[] = {isc_info_sql_stmt_type};
         char res_buffer[8];
@@ -154,26 +157,26 @@ namespace
         ISC_STATUS stat[stat_size];
 
         if (isc_dsql_sql_info(stat, &stmt, sizeof(type_item),
-                              type_item, sizeof(res_buffer), res_buffer))
+            type_item, sizeof(res_buffer), res_buffer))
         {
-            throwISCError(stat);
+            throw_iscerror(stat);
         }
 
         if (res_buffer[0] == isc_info_sql_stmt_type)
         {
             length = isc_vax_integer(res_buffer+1, 2);
-            statement_type = isc_vax_integer(res_buffer+3, length);
+            stype = isc_vax_integer(res_buffer+3, static_cast<short>(length));
         }
         else
         {
-            throw SOCIError("Can't determine statement type.");
+            throw soci_error("Can't determine statement type.");
         }
 
-        return statement_type;
+        return stype;
     }
 }
 
-void FirebirdStatementBackEnd::rewriteQuery(
+void firebird_statement_backend::rewriteQuery(
     std::string const &query, std::vector<char> &buffer)
 {
     // buffer for temporary query
@@ -219,14 +222,14 @@ void FirebirdStatementBackEnd::rewriteQuery(
     // allocate temporary statement to determine its type
     if (isc_dsql_allocate_statement(stat, &session_.dbhp_, &tmpStmtp))
     {
-        throwISCError(stat);
+        throw_iscerror(stat);
     }
 
     // prepare temporary statement
-    if (isc_dsql_prepare(stat, &(session_.trhp_), &tmpStmtp, 0,
-                         &tmpQuery[0], SQL_DIALECT_V6, sqldap_))
+    if (isc_dsql_prepare(stat, session_.current_transaction(), &tmpStmtp, 0,
+        &tmpQuery[0], SQL_DIALECT_V6, sqldap_))
     {
-        throwISCError(stat);
+        throw_iscerror(stat);
     }
 
     // get statement type
@@ -235,7 +238,7 @@ void FirebirdStatementBackEnd::rewriteQuery(
     // free temporary prepared statement
     if (isc_dsql_free_statement(stat, &tmpStmtp, DSQL_drop))
     {
-        throwISCError(stat);
+        throw_iscerror(stat);
     }
 
     // take care of special cases
@@ -283,9 +286,10 @@ void FirebirdStatementBackEnd::rewriteQuery(
     procedure_ = false;
 }
 
-void FirebirdStatementBackEnd::prepare(std::string const & query,
-                                       eStatementType /* eType */)
+void firebird_statement_backend::prepare(std::string const & query,
+                                         statement_type /* eType */)
 {
+    //std::cerr << "prepare: query=" << query << std::endl;
     // clear named parametes
     names_.clear();
 
@@ -298,20 +302,20 @@ void FirebirdStatementBackEnd::prepare(std::string const & query,
     ISC_STATUS stat[stat_size];
 
     // prepare real statement
-    if (isc_dsql_prepare(stat, &(session_.trhp_), &stmtp_, 0,
-                         &queryBuffer[0], SQL_DIALECT_V6, sqldap_))
+    if (isc_dsql_prepare(stat, session_.current_transaction(), &stmtp_, 0,
+        &queryBuffer[0], SQL_DIALECT_V6, sqldap_))
     {
-        throwISCError(stat);
+        throw_iscerror(stat);
     }
 
     if (sqldap_->sqln < sqldap_->sqld)
     {
         // sqlda is too small for all columns. it must be reallocated
-        prepareSQLDA(&sqldap_,sqldap_->sqld);
+        prepareSQLDA(&sqldap_, sqldap_->sqld);
 
         if (isc_dsql_describe(stat, &stmtp_, SQL_DIALECT_V6, sqldap_))
         {
-            throwISCError(stat);
+            throw_iscerror(stat);
         }
     }
 
@@ -323,7 +327,7 @@ void FirebirdStatementBackEnd::prepare(std::string const & query,
 
     if (isc_dsql_describe_bind(stat, &stmtp_, SQL_DIALECT_V6, sqlda2p_))
     {
-        throwISCError(stat);
+        throw_iscerror(stat);
     }
 
     if (sqlda2p_->sqln < sqlda2p_->sqld)
@@ -333,7 +337,7 @@ void FirebirdStatementBackEnd::prepare(std::string const & query,
 
         if (isc_dsql_describe_bind(stat, &stmtp_, SQL_DIALECT_V6, sqlda2p_))
         {
-            throwISCError(stat);
+            throw_iscerror(stat);
         }
     }
 
@@ -354,20 +358,20 @@ void FirebirdStatementBackEnd::prepare(std::string const & query,
 namespace
 {
     void checkSize(std::size_t actual, std::size_t expected,
-                   std::string const & name)
+        std::string const & name)
     {
         if (actual != expected)
         {
             std::ostringstream msg;
             msg << "Incorrect number of " << name << " variables. "
-            << "Expected " << expected << ", got " << actual;
-            throw SOCIError(msg.str());
+                << "Expected " << expected << ", got " << actual;
+            throw soci_error(msg.str());
         }
     }
 }
 
-StatementBackEnd::execFetchResult
-FirebirdStatementBackEnd::execute(int number)
+statement_backend::exec_fetch_result
+firebird_statement_backend::execute(int number)
 {
     ISC_STATUS stat[stat_size];
     XSQLDA *t = NULL;
@@ -388,7 +392,7 @@ FirebirdStatementBackEnd::execute(int number)
         {
             for (std::size_t col=0; col<usize; ++col)
             {
-                static_cast<FirebirdStandardUseTypeBackEnd*>(uses_[col])->exchangeData();
+                static_cast<firebird_standard_use_type_backend*>(uses_[col])->exchangeData();
             }
         }
     }
@@ -397,45 +401,57 @@ FirebirdStatementBackEnd::execute(int number)
     if (isc_dsql_free_statement(stat, &stmtp_, DSQL_close))
     {
         // ignore attempt to close already closed cursor
-        if (!checkISCError(stat, isc_dsql_cursor_close_err))
+        if (check_iscerror(stat, isc_dsql_cursor_close_err) == false)
         {
-            throwISCError(stat);
+            throw_iscerror(stat);
         }
     }
 
     if (useType_ == eVector)
     {
+        long long rowsAffectedBulkTemp = 0;
+
         // Here we have to explicitly loop to achieve the
         // effect of inserting or updating with vector use elements.
-        std::size_t rows = static_cast<FirebirdVectorUseTypeBackEnd*>(uses_[0])->size();
+        std::size_t rows = static_cast<firebird_vector_use_type_backend*>(uses_[0])->size();
         for (std::size_t row=0; row < rows; ++row)
         {
             // first we have to prepare input parameters
             for (std::size_t col=0; col<usize; ++col)
             {
-                static_cast<FirebirdVectorUseTypeBackEnd*>(uses_[col])->exchangeData(row);
+                static_cast<firebird_vector_use_type_backend*>(uses_[col])->exchangeData(row);
             }
 
             // then execute query
-            if (isc_dsql_execute(stat, &session_.trhp_, &stmtp_, SQL_DIALECT_V6, t))
+            if (isc_dsql_execute(stat, session_.current_transaction(), &stmtp_, SQL_DIALECT_V6, t))
             {
-                throwISCError(stat);
+                // preserve the number of rows affected so far.
+                rowsAffectedBulk_ = rowsAffectedBulkTemp;
+                throw_iscerror(stat);
             }
-
-            // SOCI does not allow bulk insert/update and bulk select operations
+            else
+            {
+                rowsAffectedBulkTemp += get_affected_rows();
+            }
+            // soci does not allow bulk insert/update and bulk select operations
             // in same query. So here, we know that into elements are not
             // vectors. So, there is no need to fetch data here.
         }
+        rowsAffectedBulk_ = rowsAffectedBulkTemp;
     }
     else
     {
         // use elements aren't vectors
-        if (isc_dsql_execute(stat, &session_.trhp_, &stmtp_, SQL_DIALECT_V6, t))
+        if (isc_dsql_execute(stat, session_.current_transaction(), &stmtp_, SQL_DIALECT_V6, t))
         {
-            throwISCError(stat);
+            throw_iscerror(stat);
         }
     }
 
+    // Successfully re-executing the statement must reset the "end of rowset"
+    // flag, we might be able to fetch data again now.
+    endOfRowSet_ = false;
+
     if (sqldap_->sqld)
     {
         // query may return some data
@@ -447,19 +463,22 @@ FirebirdStatementBackEnd::execute(int number)
         else
         {
             // execute(0) was meant to only perform the query
-            return eSuccess;
+            return ef_success;
         }
     }
     else
     {
         // query can't return any data
-        return eNoData;
+        return ef_no_data;
     }
 }
 
-StatementBackEnd::execFetchResult
-FirebirdStatementBackEnd::fetch(int number)
+statement_backend::exec_fetch_result
+firebird_statement_backend::fetch(int number)
 {
+    if (endOfRowSet_)
+        return ef_no_data;
+
     ISC_STATUS stat[stat_size];
 
     for (size_t i = 0; i<static_cast<unsigned int>(sqldap_->sqld); ++i)
@@ -473,7 +492,8 @@ FirebirdStatementBackEnd::fetch(int number)
     rowsFetched_ = 0;
     for (int i = 0; i < number; ++i)
     {
-        long fetch_stat = isc_dsql_fetch(stat, &stmtp_, SQL_DIALECT_V6, sqldap_);
+        ISC_STATUS const
+            fetch_stat = isc_dsql_fetch(stat, &stmtp_, SQL_DIALECT_V6, sqldap_);
 
         // there is more data to read
         if (fetch_stat == 0)
@@ -483,66 +503,58 @@ FirebirdStatementBackEnd::fetch(int number)
         }
         else if (fetch_stat == 100L)
         {
-            return eNoData;
+            endOfRowSet_ = true;
+            return ef_no_data;
         }
         else
         {
             // error
-            throwISCError(stat);
-            return eNoData; // unreachable, for compiler only
+            endOfRowSet_ = true;
+            throw_iscerror(stat);
+            return ef_no_data; // unreachable, for compiler only
         }
     } // for
 
-    return eSuccess;
+    return ef_success;
 }
 
 // here we put data fetched from database into user buffers
-void FirebirdStatementBackEnd::exchangeData(bool gotData, int row)
+void firebird_statement_backend::exchangeData(bool gotData, int row)
 {
-    // first save indicators
-    for (size_t i = 0; i < static_cast<unsigned int>(sqldap_->sqld); ++i)
+    if (gotData)
     {
-        if (!gotData)
-        {
-            inds_[i][row] = eIndicator(eNoData);
-        }
-        else
+        for (size_t i = 0; i < static_cast<unsigned int>(sqldap_->sqld); ++i)
         {
+            // first save indicators
             if (((sqldap_->sqlvar+i)->sqltype & 1) == 0)
             {
                 // there is no indicator for this column
-                inds_[i][row] = eOK;
+                inds_[i][row] = i_ok;
             }
             else if (*((sqldap_->sqlvar+i)->sqlind) == 0)
             {
-                inds_[i][row] = eOK;
+                inds_[i][row] = i_ok;
             }
             else if (*((sqldap_->sqlvar+i)->sqlind) == -1)
             {
-                inds_[i][row] = eNull;
+                inds_[i][row] = i_null;
             }
             else
             {
-                throw SOCIError("Unknown state in FirebirdStatementBackEnd::exchangeData()");
+                throw soci_error("Unknown state in firebird_statement_backend::exchangeData()");
             }
-        }
-    }
 
-    // then deal with data
-    if (gotData)
-    {
-        for (size_t i = 0; i<static_cast<unsigned int>(sqldap_->sqld); ++i)
-        {
-            if (inds_[i][row] != eNull)
+            // then deal with data
+            if (inds_[i][row] != i_null)
             {
                 if (intoType_ == eVector)
                 {
-                    static_cast<FirebirdVectorIntoTypeBackEnd*>(
+                    static_cast<firebird_vector_into_type_backend*>(
                         intos_[i])->exchangeData(row);
                 }
                 else
                 {
-                    static_cast<FirebirdStandardIntoTypeBackEnd*>(
+                    static_cast<firebird_standard_into_type_backend*>(
                         intos_[i])->exchangeData();
                 }
             }
@@ -550,25 +562,103 @@ void FirebirdStatementBackEnd::exchangeData(bool gotData, int row)
     }
 }
 
-int FirebirdStatementBackEnd::getNumberOfRows()
+long long firebird_statement_backend::get_affected_rows()
+{
+    if (rowsAffectedBulk_ >= 0)
+    {
+        return rowsAffectedBulk_;
+    }
+
+    ISC_STATUS_ARRAY stat;
+    char type_item[] = { isc_info_sql_records };
+    char res_buffer[256];
+
+    if (isc_dsql_sql_info(stat, &stmtp_, sizeof(type_item), type_item,
+                          sizeof(res_buffer), res_buffer))
+    {
+        throw_iscerror(stat);
+    }
+
+    // We must get back a isc_info_sql_records block, that we parse below,
+    // followed by isc_info_end.
+    if (res_buffer[0] != isc_info_sql_records)
+    {
+        throw soci_error("Can't determine the number of affected rows");
+    }
+
+    char* sql_rec_buf = res_buffer + 1;
+    const int length = isc_vax_integer(sql_rec_buf, 2);
+    sql_rec_buf += 2;
+
+    if (sql_rec_buf[length] != isc_info_end)
+    {
+        throw soci_error("Unexpected isc_info_sql_records return format");
+    }
+
+    // Examine the 4 sub-blocks each of which has a header indicating the block
+    // type, its value length in bytes and the value itself.
+    long long row_count = 0;
+
+    for ( char* p = sql_rec_buf; !row_count && p < sql_rec_buf + length; )
+    {
+        switch (*p++)
+        {
+            case isc_info_req_select_count:
+            case isc_info_req_insert_count:
+            case isc_info_req_update_count:
+            case isc_info_req_delete_count:
+                {
+                    int len = isc_vax_integer(p, 2);
+                    p += 2;
+
+                    row_count += isc_vax_integer(p, static_cast<short>(len));
+                    p += len;
+                }
+                break;
+
+            case isc_info_end:
+                break;
+
+            default:
+                throw soci_error("Unknown record counter");
+        }
+    }
+
+    return row_count;
+}
+
+int firebird_statement_backend::get_number_of_rows()
 {
     return rowsFetched_;
 }
 
-std::string FirebirdStatementBackEnd::rewriteForProcedureCall(
+std::string firebird_statement_backend::get_parameter_name(int index) const
+{
+    for (std::map<std::string, int>::const_iterator i = names_.begin();
+         i != names_.end();
+         ++i)
+    {
+        if (i->second == index)
+            return i->first;
+    }
+
+    return std::string();
+}
+
+std::string firebird_statement_backend::rewrite_for_procedure_call(
     std::string const &query)
 {
     procedure_ = true;
     return query;
 }
 
-int FirebirdStatementBackEnd::prepareForDescribe()
+int firebird_statement_backend::prepare_for_describe()
 {
     return static_cast<int>(sqldap_->sqld);
 }
 
-void FirebirdStatementBackEnd::describeColumn(int colNum,
-        eDataType & type, std::string & columnName)
+void firebird_statement_backend::describe_column(int colNum,
+                                                data_type & type, std::string & columnName)
 {
     XSQLVAR * var = sqldap_->sqlvar+(colNum-1);
 
@@ -576,71 +666,73 @@ void FirebirdStatementBackEnd::describeColumn(int colNum,
 
     switch (var->sqltype & ~1)
     {
-        case SQL_TEXT:
-        case SQL_VARYING:
-            type = eString;
-            break;
-        case SQL_TYPE_DATE:
-        case SQL_TYPE_TIME:
-        case SQL_TIMESTAMP:
-            type = eDate;
-            break;
-        case SQL_FLOAT:
-        case SQL_DOUBLE:
-            type = eDouble;
-            break;
-        case SQL_SHORT:
-        case SQL_LONG:
-            if (var->sqlscale < 0)
-            {
-                type = eDouble;
-            }
+    case SQL_TEXT:
+    case SQL_VARYING:
+        type = dt_string;
+        break;
+    case SQL_TYPE_DATE:
+    case SQL_TYPE_TIME:
+    case SQL_TIMESTAMP:
+        type = dt_date;
+        break;
+    case SQL_FLOAT:
+    case SQL_DOUBLE:
+        type = dt_double;
+        break;
+    case SQL_SHORT:
+    case SQL_LONG:
+        if (var->sqlscale < 0)
+        {
+            if (session_.get_option_decimals_as_strings())
+                type = dt_string;
             else
-            {
-                type = eInteger;
-            }
-            break;
-        case SQL_INT64:
-            if (var->sqlscale < 0)
-            {
-                type = eDouble;
-            }
+                type = dt_double;
+        }
+        else
+        {
+            type = dt_integer;
+        }
+        break;
+    case SQL_INT64:
+        if (var->sqlscale < 0)
+        {
+            if (session_.get_option_decimals_as_strings())
+                type = dt_string;
             else
-            {
-                // 64bit integers are not supported
-                std::ostringstream msg;
-                msg << "Type of column ["<< colNum << "] \"" << columnName
-                << "\" is not supported for dynamic queries";
-                throw SOCIError(msg.str());
-            }
-            break;
-            /* case SQL_BLOB:
-            case SQL_ARRAY:*/
-        default:
-            std::ostringstream msg;
-            msg << "Type of column ["<< colNum << "] \"" << columnName
+                type = dt_double;
+        }
+        else
+        {
+            type = dt_long_long;
+        }
+        break;
+        /* case SQL_BLOB:
+        case SQL_ARRAY:*/
+    default:
+        std::ostringstream msg;
+        msg << "Type of column ["<< colNum << "] \"" << columnName
             << "\" is not supported for dynamic queries";
-            throw SOCIError(msg.str());
-            break;
+        throw soci_error(msg.str());
+        break;
     }
 }
 
-FirebirdStandardIntoTypeBackEnd * FirebirdStatementBackEnd::makeIntoTypeBackEnd()
+firebird_standard_into_type_backend * firebird_statement_backend::make_into_type_backend()
 {
-    return new FirebirdStandardIntoTypeBackEnd(*this);
+    return new firebird_standard_into_type_backend(*this);
 }
 
-FirebirdStandardUseTypeBackEnd * FirebirdStatementBackEnd::makeUseTypeBackEnd()
+firebird_standard_use_type_backend * firebird_statement_backend::make_use_type_backend()
 {
-    return new FirebirdStandardUseTypeBackEnd(*this);
+    return new firebird_standard_use_type_backend(*this);
 }
 
-FirebirdVectorIntoTypeBackEnd * FirebirdStatementBackEnd::makeVectorIntoTypeBackEnd()
+firebird_vector_into_type_backend * firebird_statement_backend::make_vector_into_type_backend()
 {
-    return new FirebirdVectorIntoTypeBackEnd(*this);
+    return new firebird_vector_into_type_backend(*this);
 }
 
-FirebirdVectorUseTypeBackEnd * FirebirdStatementBackEnd::makeVectorUseTypeBackEnd()
+firebird_vector_use_type_backend * firebird_statement_backend::make_vector_use_type_backend()
 {
-    return new FirebirdVectorUseTypeBackEnd(*this);
+    return new firebird_vector_use_type_backend(*this);
 }
