@@ -30,6 +30,33 @@ using std::string;
 namespace
 { // anonymous
 
+// Helper class used to ensure we call mysql_library_init() before opening the
+// first MySQL connection and mysql_library_end() on application shutdown.
+class mysql_library
+{
+private:
+    mysql_library()
+    {
+        if (mysql_library_init(0, NULL, NULL))
+        {
+            throw soci_error("Failed to initialize MySQL library.");
+        }
+    }
+
+    ~mysql_library()
+    {
+        mysql_library_end();
+    }
+
+public:
+    // This function only exists for its side effect of creating an instance of
+    // this class.
+    static void ensure_initialized()
+    {
+        static mysql_library ins;
+    }
+};
+
 void skip_white(std::string::const_iterator *i,
     std::string::const_iterator const & end, bool endok)
 {
@@ -170,7 +197,7 @@ void parse_connect_string(const string & connectString,
     int *port, bool *port_p, string *ssl_ca, bool *ssl_ca_p,
     string *ssl_cert, bool *ssl_cert_p, string *ssl_key, bool *ssl_key_p,
     int *local_infile, bool *local_infile_p,
-    string *charset, bool *charset_p,
+    string *charset, bool *charset_p, bool *reconnect_p,
     unsigned int *connect_timeout, bool *connect_timeout_p,
     unsigned int *read_timeout, bool *read_timeout_p,
     unsigned int *write_timeout, bool *write_timeout_p)
@@ -186,6 +213,7 @@ void parse_connect_string(const string & connectString,
     *ssl_key_p = false;
     *local_infile_p = false;
     *charset_p = false;
+    *reconnect_p = false;
     *connect_timeout_p = false;
     *read_timeout_p = false;
     *write_timeout_p = false;
@@ -280,26 +308,32 @@ void parse_connect_string(const string & connectString,
         {
             *charset = val;
             *charset_p = true;
+        } else if (par == "reconnect" && !*reconnect_p)
+        {
+            if (val != "1")
+                throw soci_error("\"reconnect\" option may only be set to 1");
+
+            *reconnect_p = true;
         } else if (par == "connect_timeout" && !*connect_timeout_p)
         {
             if (!valid_uint(val))
                 throw soci_error(err);
-            char *end;
-            *connect_timeout = std::strtoul(val.c_str(), &end, 10);
+            char *endp;
+            *connect_timeout = std::strtoul(val.c_str(), &endp, 10);
             *connect_timeout_p = true;
         } else if (par == "read_timeout" && !*read_timeout_p)
         {
             if (!valid_uint(val))
                 throw soci_error(err);
-            char *end;
-            *read_timeout = std::strtoul(val.c_str(), &end, 10);
+            char *endp;
+            *read_timeout = std::strtoul(val.c_str(), &endp, 10);
             *read_timeout_p = true;
         } else if (par == "write_timeout" && !*write_timeout_p)
         {
             if (!valid_uint(val))
                 throw soci_error(err);
-            char *end;
-            *write_timeout = std::strtoul(val.c_str(), &end, 10);
+            char *endp;
+            *write_timeout = std::strtoul(val.c_str(), &endp, 10);
             *write_timeout_p = true;
         }
         else
@@ -326,18 +360,20 @@ void parse_connect_string(const string & connectString,
 mysql_session_backend::mysql_session_backend(
     connection_parameters const & parameters)
 {
+    mysql_library::ensure_initialized();
+
     string host, user, password, db, unix_socket, ssl_ca, ssl_cert, ssl_key,
         charset;
     int port, local_infile;
     unsigned int connect_timeout, read_timeout, write_timeout;
     bool host_p, user_p, password_p, db_p, unix_socket_p, port_p,
-        ssl_ca_p, ssl_cert_p, ssl_key_p, local_infile_p, charset_p,
+        ssl_ca_p, ssl_cert_p, ssl_key_p, local_infile_p, charset_p, reconnect_p,
         connect_timeout_p, read_timeout_p, write_timeout_p;
     parse_connect_string(parameters.get_connect_string(), &host, &host_p, &user, &user_p,
         &password, &password_p, &db, &db_p,
         &unix_socket, &unix_socket_p, &port, &port_p,
         &ssl_ca, &ssl_ca_p, &ssl_cert, &ssl_cert_p, &ssl_key, &ssl_key_p,
-        &local_infile, &local_infile_p, &charset, &charset_p,
+        &local_infile, &local_infile_p, &charset, &charset_p, &reconnect_p,
         &connect_timeout, &connect_timeout_p,
         &read_timeout, &read_timeout_p,
         &write_timeout, &write_timeout_p);
@@ -346,6 +382,19 @@ mysql_session_backend::mysql_session_backend(
     {
         throw soci_error("mysql_init() failed.");
     }
+    if (reconnect_p)
+    {
+        #if MYSQL_VERSION_ID < 8
+            my_bool reconnect = 1;
+        #else
+            bool reconnect = 1;
+        #endif
+        if (0 != mysql_options(conn_, MYSQL_OPT_RECONNECT, &reconnect))
+        {
+            clean_up();
+            throw soci_error("mysql_options(MYSQL_OPT_RECONNECT) failed.");
+        }
+    }
     if (charset_p)
     {
         if (0 != mysql_options(conn_, MYSQL_SET_CHARSET_NAME, charset.c_str()))
