@@ -199,7 +199,7 @@ void mysql_vector_into_type_backend::resize(std::size_t sz)
 
 std::size_t mysql_vector_into_type_backend::size()
 {
-    std::size_t sz = 0; // dummy initialization to please the compiler
+    std::size_t sz SOCI_DUMMY_INIT(0);
     switch (type_)
     {
         // simple cases
