@@ -185,7 +185,7 @@ void mysql_vector_use_type_backend::pre_use(indicator const *ind)
 
 std::size_t mysql_vector_use_type_backend::size()
 {
-    std::size_t sz = 0; // dummy initialization to please the compiler
+    std::size_t sz SOCI_DUMMY_INIT(0);
     switch (type_)
     {
         // simple cases
