@@ -6,38 +6,37 @@
 //
 
 #define SOCI_ODBC_SOURCE
-#include "soci-odbc.h"
-#include <soci.h>
+#include "soci/odbc/soci-odbc.h"
 
-using namespace SOCI;
-using namespace SOCI::details;
+using namespace soci;
+using namespace soci::details;
 
 
-ODBCBLOBBackEnd::ODBCBLOBBackEnd(ODBCSessionBackEnd &session)
+odbc_blob_backend::odbc_blob_backend(odbc_session_backend &session)
     : session_(session)
 {
     // ...
 }
 
-ODBCBLOBBackEnd::~ODBCBLOBBackEnd()
+odbc_blob_backend::~odbc_blob_backend()
 {
     // ...
 }
 
-std::size_t ODBCBLOBBackEnd::getLen()
+std::size_t odbc_blob_backend::get_len()
 {
     // ...
     return 0;
 }
 
-std::size_t ODBCBLOBBackEnd::read(
+std::size_t odbc_blob_backend::read(
     std::size_t /* offset */, char * /* buf */, std::size_t /* toRead */)
 {
     // ...
     return 0;
 }
 
-std::size_t ODBCBLOBBackEnd::write(
+std::size_t odbc_blob_backend::write(
     std::size_t /* offset */, char const * /* buf */,
     std::size_t /* toWrite */)
 {
@@ -45,14 +44,14 @@ std::size_t ODBCBLOBBackEnd::write(
     return 0;
 }
 
-std::size_t ODBCBLOBBackEnd::append(
+std::size_t odbc_blob_backend::append(
     char const * /* buf */, std::size_t /* toWrite */)
 {
     // ...
     return 0;
 }
 
-void ODBCBLOBBackEnd::trim(std::size_t /* newLen */)
+void odbc_blob_backend::trim(std::size_t /* newLen */)
 {
     // ...
 }
