@@ -16,7 +16,7 @@ using namespace soci::details;
 
 
 odbc_statement_backend::odbc_statement_backend(odbc_session_backend &session)
-    : session_(session), hstmt_(0), numRowsFetched_(0),
+    : session_(session), hstmt_(0), numRowsFetched_(0), fetchVectorByRows_(false),
       hasVectorUseElements_(false), boundByName_(false), boundByPos_(false),
       rowsAffected_(-1LL)
 {
@@ -131,6 +131,10 @@ void odbc_statement_backend::prepare(std::string const & query,
         ss << "preparing query \"" << query_ << "\"";
         throw odbc_soci_error(SQL_HANDLE_STMT, hstmt_, ss.str());
     }
+
+    // reset any old into buffers, they will be added later if they're used
+    // with this query
+    intos_.clear();
 }
 
 statement_backend::exec_fetch_result
@@ -211,15 +215,8 @@ odbc_statement_backend::execute(int number)
 }
 
 statement_backend::exec_fetch_result
-odbc_statement_backend::fetch(int number)
+odbc_statement_backend::do_fetch(int beginRow, int endRow)
 {
-    numRowsFetched_ = 0;
-    SQLULEN const row_array_size = static_cast<SQLULEN>(number);
-
-    SQLSetStmtAttr(hstmt_, SQL_ATTR_ROW_BIND_TYPE, SQL_BIND_BY_COLUMN, 0);
-    SQLSetStmtAttr(hstmt_, SQL_ATTR_ROW_ARRAY_SIZE, (SQLPOINTER)row_array_size, 0);
-    SQLSetStmtAttr(hstmt_, SQL_ATTR_ROWS_FETCHED_PTR, &numRowsFetched_, 0);
-
     SQLRETURN rc = SQLFetch(hstmt_);
 
     if (SQL_NO_DATA == rc)
@@ -232,9 +229,72 @@ odbc_statement_backend::fetch(int number)
         throw odbc_soci_error(SQL_HANDLE_STMT, hstmt_, "fetching data");
     }
 
+    for (std::size_t j = 0; j != intos_.size(); ++j)
+    {
+        intos_[j]->do_post_fetch_rows(beginRow, endRow);
+    }
+
     return ef_success;
 }
 
+statement_backend::exec_fetch_result
+odbc_statement_backend::fetch(int number)
+{
+    numRowsFetched_ = 0;
+
+    for (std::size_t i = 0; i != intos_.size(); ++i)
+    {
+        intos_[i]->resize(number);
+    }
+
+    SQLSetStmtAttr(hstmt_, SQL_ATTR_ROW_BIND_TYPE, SQL_BIND_BY_COLUMN, 0);
+
+    statement_backend::exec_fetch_result res SOCI_DUMMY_INIT(ef_success);
+
+    // Usually we try to fetch the entire vector at once, but if some into
+    // string columns are bigger than 8KB (ODBC_MAX_COL_SIZE) then we use
+    // 100MB buffer for that columns. So in this case we downgrade to using
+    // scalar fetches to hold the buffer only for a single row and not
+    // rows_count * 100MB.
+    // See odbc_vector_into_type_backend::define_by_pos().
+    if (!fetchVectorByRows_)
+    {
+        SQLULEN row_array_size = static_cast<SQLULEN>(number);
+        SQLSetStmtAttr(hstmt_, SQL_ATTR_ROW_ARRAY_SIZE, (SQLPOINTER)row_array_size, 0);
+
+        SQLSetStmtAttr(hstmt_, SQL_ATTR_ROWS_FETCHED_PTR, &numRowsFetched_, 0);
+
+        res = do_fetch(0, number);
+    }
+    else // Use multiple calls to SQLFetch().
+    {
+        SQLULEN curNumRowsFetched = 0;
+        SQLSetStmtAttr(hstmt_, SQL_ATTR_ROWS_FETCHED_PTR, &curNumRowsFetched, 0);
+
+        for (int row = 0; row < number; ++row)
+        {
+            // Unfortunately we need to redefine all vector intos which
+            // were bound to the first element of the vector initially.
+            //
+            // Note that we need to do it even for row == 0 as this might not
+            // be the first call to fetch() and so the current bindings might
+            // not be the same as initial ones.
+            for (std::size_t j = 0; j != intos_.size(); ++j)
+            {
+                intos_[j]->rebind_row(row);
+            }
+
+            res = do_fetch(row, row + 1);
+            if (res != ef_success)
+                break;
+
+            numRowsFetched_ += curNumRowsFetched;
+        }
+    }
+
+    return res;
+}
+
 long long odbc_statement_backend::get_affected_rows()
 {
     return rowsAffected_;
@@ -259,7 +319,11 @@ std::string odbc_statement_backend::rewrite_for_procedure_call(
 int odbc_statement_backend::prepare_for_describe()
 {
     SQLSMALLINT numCols;
-    SQLNumResultCols(hstmt_, &numCols);
+    SQLRETURN rc = SQLNumResultCols(hstmt_, &numCols);
+    if (is_odbc_error(rc))
+    {
+        throw soci_error("Failed to get result columns count");
+    }
     return numCols;
 }
 
