@@ -5,13 +5,13 @@
 // http://www.boost.org/LICENSE_1_0.txt)
 //
 
-#ifndef UTILITY_H_INCLUDED
-#define UTILITY_H_INCLUDED
+#ifndef SOCI_UTILITY_H_INCLUDED
+#define SOCI_UTILITY_H_INCLUDED
 
 #include "soci-backend.h"
 #include <sstream>
 
-namespace SOCI
+namespace soci
 {
 
 inline void throw_odbc_error(SQLSMALLINT htype, SQLHANDLE hndl, char const * msg)
@@ -26,7 +26,7 @@ inline void throw_odbc_error(SQLSMALLINT htype, SQLHANDLE hndl, char const * msg
     i = 1;
 
     /* get multiple field settings of diagnostic record */
-    while (SQLGetDiagRec(htype,
+    while (SQLGetDiagRecA(htype,
                          hndl,
                          i,
                          sqlstate,
@@ -35,25 +35,28 @@ inline void throw_odbc_error(SQLSMALLINT htype, SQLHANDLE hndl, char const * msg
                          SQL_MAX_MESSAGE_LENGTH + 1,
                          &length) == SQL_SUCCESS)
     {
-        ss << std::endl << "SOCI ODBC Error: " << msg << std::endl 
+        ss << std::endl << "SOCI ODBC Error: " << msg << std::endl
            << "SQLSTATE = " << sqlstate << std::endl
            << "Native Error Code = " << sqlcode << std::endl
            << message << std::endl;
         ++i;
     }
 
-    throw SOCIError(ss.str());
+    throw soci_error(ss.str());
 }
 
 inline bool is_odbc_error(SQLRETURN rc)
 {
     if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
+    {
         return true;
+    }
     else
+    {
         return false;
+    }
 }
 
 }
 
-#endif
-
+#endif // SOCI_UTILITY_H_INCLUDED
