@@ -8,6 +8,7 @@
 #define SOCI_ORACLE_SOURCE
 #include "soci/oracle/soci-oracle.h"
 #include "soci/callbacks.h"
+#include "soci/session.h"
 #include "error.h"
 #include <cctype>
 #include <cstdio>
@@ -33,16 +34,16 @@ sb4 fo_callback(void * /* svchp */, void * /* envhp */, void * fo_ctx,
         static_cast<oracle_session_backend *>(fo_ctx);
 
     failover_callback * callback = backend->failoverCallback_;
-    
+
     if (callback != NULL)
     {
         session * sql = backend->session_;
-        
+
         switch (fo_event)
         {
         case OCI_FO_BEGIN:
             // failover operation was initiated
-    
+
             try
             {
                 callback->started();
@@ -51,12 +52,12 @@ sb4 fo_callback(void * /* svchp */, void * /* envhp */, void * fo_ctx,
             {
                 // ignore exceptions from user callbacks
             }
-            
+
             break;
-            
+
         case OCI_FO_END:
             // failover was successful
-            
+
             try
             {
                 callback->finished(*sql);
@@ -67,7 +68,7 @@ sb4 fo_callback(void * /* svchp */, void * /* envhp */, void * fo_ctx,
             }
 
             break;
-            
+
         case OCI_FO_ABORT:
             // failover was aborted with no possibility to recovery
 
@@ -84,16 +85,16 @@ sb4 fo_callback(void * /* svchp */, void * /* envhp */, void * fo_ctx,
 
         case OCI_FO_ERROR:
             // failover failed, but can be retried
-            
+
             try
             {
                 bool retry = false;
                 std::string newTarget;
                 callback->failed(retry, newTarget);
-                
+
                 // newTarget is ignored, as the new target
                 // is selected by Oracle client configuration
-                
+
                 if (retry)
                 {
                     return OCI_FO_RETRY;
@@ -109,7 +110,7 @@ sb4 fo_callback(void * /* svchp */, void * /* envhp */, void * fo_ctx,
         case OCI_FO_REAUTH:
             // nothing interesting
             break;
-            
+
         default:
             // ignore unknown callback types (if any)
             break;
@@ -146,7 +147,7 @@ oracle_session_backend::oracle_session_backend(std::string const & serviceName,
     size_t nlsUserNameLen;
     char nlsPassword[authBufLen];
     size_t nlsPasswordLen;
-    
+
     sword res;
 
     // create the environment
@@ -178,7 +179,7 @@ oracle_session_backend::oracle_session_backend(std::string const & serviceName,
     if (charset != 0)
     {
         // convert service/user/password to the expected charset
-        
+
         res = OCINlsCharSetConvert(envhp_, errhp_,
             charset, nlsService, serviceBufLen,
             defaultSourceCharSetId, serviceName.c_str(), serviceName.size(), &nlsServiceLen);
@@ -190,7 +191,7 @@ oracle_session_backend::oracle_session_backend(std::string const & serviceName,
             clean_up();
             throw oracle_soci_error(msg, errNum);
         }
-        
+
         res = OCINlsCharSetConvert(envhp_, errhp_,
             charset, nlsUserName, authBufLen,
             defaultSourceCharSetId, userName.c_str(), userName.size(), &nlsUserNameLen);
@@ -202,7 +203,7 @@ oracle_session_backend::oracle_session_backend(std::string const & serviceName,
             clean_up();
             throw oracle_soci_error(msg, errNum);
         }
-        
+
         res = OCINlsCharSetConvert(envhp_, errhp_,
             charset, nlsPassword, authBufLen,
             defaultSourceCharSetId, password.c_str(), password.size(), &nlsPasswordLen);
@@ -218,7 +219,7 @@ oracle_session_backend::oracle_session_backend(std::string const & serviceName,
     else
     {
         // do not perform any charset conversions
-        
+
         nlsServiceLen = serviceName.size();
         if (nlsServiceLen < serviceBufLen)
         {
@@ -249,10 +250,11 @@ oracle_session_backend::oracle_session_backend(std::string const & serviceName,
             throw soci_error("Password is too long.");
         }
     }
-    
+
     // create the server context
     res = OCIServerAttach(srvhp_, errhp_,
-        reinterpret_cast<text*>(nlsService), nlsServiceLen, OCI_DEFAULT);
+        reinterpret_cast<text*>(nlsService),
+        static_cast<sb4>(nlsServiceLen), OCI_DEFAULT);
     if (res != OCI_SUCCESS)
     {
         std::string msg;
@@ -309,7 +311,7 @@ oracle_session_backend::oracle_session_backend(std::string const & serviceName,
     }
 
     // select credentials type - use rdbms based credentials by default
-    // and switch to external credentials if username and 
+    // and switch to external credentials if username and
     // password are both not specified
     ub4 credentialType = OCI_CRED_RDBMS;
     if (userName.empty() && password.empty())
@@ -321,7 +323,7 @@ oracle_session_backend::oracle_session_backend(std::string const & serviceName,
         // set username attribute in the user session handle
         res = OCIAttrSet(usrhp_, OCI_HTYPE_SESSION,
             reinterpret_cast<dvoid*>(nlsUserName),
-            nlsUserNameLen, OCI_ATTR_USERNAME, errhp_);
+            static_cast<ub4>(nlsUserNameLen), OCI_ATTR_USERNAME, errhp_);
         if (res != OCI_SUCCESS)
         {
             clean_up();
@@ -331,7 +333,7 @@ oracle_session_backend::oracle_session_backend(std::string const & serviceName,
         // set password attribute
         res = OCIAttrSet(usrhp_, OCI_HTYPE_SESSION,
             reinterpret_cast<dvoid*>(nlsPassword),
-            nlsPasswordLen, OCI_ATTR_PASSWORD, errhp_);
+            static_cast<ub4>(nlsPasswordLen), OCI_ATTR_PASSWORD, errhp_);
         if (res != OCI_SUCCESS)
         {
             clean_up();
@@ -437,6 +439,15 @@ oracle_blob_backend * oracle_session_backend::make_blob_backend()
     return new oracle_blob_backend(*this);
 }
 
+bool oracle_session_backend::get_next_sequence_value(
+    session &s, std::string const &sequence,
+    long long &value)
+{
+    s << "select " + sequence + ".nextval from dual", into(value);
+
+    return true;
+}
+
 ub2 oracle_session_backend::get_double_sql_type() const
 {
     // SQLT_BDOUBLE avoids unnecessary conversions which is better from both
