@@ -7,8 +7,10 @@
 
 #define soci_ORACLE_SOURCE
 #include "soci/oracle/soci-oracle.h"
+#include "clob.h"
 #include "error.h"
 #include "soci/soci-platform.h"
+#include "soci-vector-helpers.h"
 #include <cctype>
 #include <cstdio>
 #include <cstring>
@@ -32,7 +34,6 @@ void oracle_vector_use_type_backend::prepare_indicators(std::size_t size)
     }
 
     indOCIHolderVec_.resize(size);
-    indOCIHolders_ = &indOCIHolderVec_[0];
 }
 
 void oracle_vector_use_type_backend::prepare_for_bind(
@@ -155,11 +156,25 @@ void oracle_vector_use_type_backend::prepare_for_bind(
         }
         break;
 
-    case x_xmltype:    break; // not supported
-    case x_longstring: break; // not supported
-    case x_statement:  break; // not supported
-    case x_rowid:      break; // not supported
-    case x_blob:       break; // not supported
+    case x_xmltype:
+    case x_longstring:
+        {
+            std::size_t const vecSize = size();
+            prepare_indicators(vecSize);
+
+            sb4 const dlen = sizeof(OCILobLocator*);
+            buf_ = new char[dlen * vecSize];
+
+            oracleType = SQLT_CLOB;
+            data = buf_;
+            elementSize = dlen;
+        }
+        break;
+
+    case x_statement:
+    case x_rowid:
+    case x_blob:
+        throw soci_error("Unsupported type for vector use parameter");
     }
 }
 
@@ -173,27 +188,8 @@ void oracle_vector_use_type_backend::bind_by_pos_bulk(int & position,
     end_ = end;
 
     end_var_ = full_size();
-    
-    ub2 oracleType;
-    sb4 elementSize;
-    void * dataBuf;
 
-    prepare_for_bind(dataBuf, elementSize, oracleType);
-
-    ub2 *sizesP = 0; // used only for std::string
-    if (type == x_stdstring)
-    {
-        sizesP = &sizes_[0];
-    }
-
-    sword res = OCIBindByPos(statement_.stmtp_, &bindp_,
-        statement_.session_.errhp_,
-        position++, dataBuf, elementSize, oracleType,
-        indOCIHolders_, sizesP, 0, 0, 0, OCI_DEFAULT);
-    if (res != OCI_SUCCESS)
-    {
-        throw_oracle_soci_error(res, statement_.session_.errhp_);
-    }
+    bind_position_ = position++;
 }
 
 void oracle_vector_use_type_backend::bind_by_name_bulk(
@@ -206,7 +202,12 @@ void oracle_vector_use_type_backend::bind_by_name_bulk(
     end_ = end;
 
     end_var_ = full_size();
-    
+
+    bind_name_ = name;
+}
+
+void oracle_vector_use_type_backend::pre_use(indicator const *ind)
+{
     ub2 oracleType;
     sb4 elementSize;
     void * dataBuf;
@@ -214,31 +215,15 @@ void oracle_vector_use_type_backend::bind_by_name_bulk(
     prepare_for_bind(dataBuf, elementSize, oracleType);
 
     ub2 *sizesP = 0; // used only for std::string
-    if (type == x_stdstring)
+    if (type_ == x_stdstring)
     {
         sizesP = &sizes_[0];
     }
 
-    sword res = OCIBindByName(statement_.stmtp_, &bindp_,
-        statement_.session_.errhp_,
-        reinterpret_cast<text*>(const_cast<char*>(name.c_str())),
-        static_cast<sb4>(name.size()),
-        dataBuf, elementSize, oracleType,
-        indOCIHolders_, sizesP, 0, 0, 0, OCI_DEFAULT);
-    if (res != OCI_SUCCESS)
-    {
-        throw_oracle_soci_error(res, statement_.session_.errhp_);
-    }
-}
-
-void oracle_vector_use_type_backend::pre_use(indicator const *ind)
-{
     // first deal with data
     if (type_ == x_stdstring)
     {
-        // nothing to do - it's already done during bind
-        // (and it's probably impossible to separate them, because
-        // changes in the string size could not be handled here)
+        // nothing to do - already done in prepare_for_bind()
     }
     else if (type_ == x_long_long)
     {
@@ -281,7 +266,7 @@ void oracle_vector_use_type_backend::pre_use(indicator const *ind)
         for (std::size_t i = 0; i != vecSize; ++i)
         {
             std::tm & t = v[begin_ + i];
-            
+
             *pos++ = static_cast<ub1>(100 + (1900 + t.tm_year) / 100);
             *pos++ = static_cast<ub1>(100 + t.tm_year % 100);
             *pos++ = static_cast<ub1>(t.tm_mon + 1);
@@ -291,6 +276,18 @@ void oracle_vector_use_type_backend::pre_use(indicator const *ind)
             *pos++ = static_cast<ub1>(t.tm_sec + 1);
         }
     }
+    else if (type_ == x_longstring || type_ == x_xmltype)
+    {
+        OCILobLocator** const lobps = reinterpret_cast<OCILobLocator**>(buf_);
+
+        std::size_t const vecSize = size();
+        for (std::size_t i = 0; i != vecSize; ++i)
+        {
+            lobps[i] = create_temp_lob(statement_.session_);
+            write_to_lob(statement_.session_, lobps[i],
+                vector_string_value(type_, data_, i));
+        }
+    }
 
     // then handle indicators
     if (ind != NULL)
@@ -317,6 +314,29 @@ void oracle_vector_use_type_backend::pre_use(indicator const *ind)
             indOCIHolderVec_[i] = 0;  // value is OK
         }
     }
+
+    sword res;
+    if (bind_name_.empty())
+    {
+        res = OCIBindByPos(statement_.stmtp_, &bindp_,
+            statement_.session_.errhp_,
+            bind_position_, dataBuf, elementSize, oracleType,
+            &indOCIHolderVec_[0], sizesP, 0, 0, 0, OCI_DEFAULT);
+    }
+    else
+    {
+        res = OCIBindByName(statement_.stmtp_, &bindp_,
+            statement_.session_.errhp_,
+            reinterpret_cast<text*>(const_cast<char*>(bind_name_.c_str())),
+            static_cast<sb4>(bind_name_.size()),
+            dataBuf, elementSize, oracleType,
+            &indOCIHolderVec_[0], sizesP, 0, 0, 0, OCI_DEFAULT);
+    }
+
+    if (res != OCI_SUCCESS)
+    {
+        throw_oracle_soci_error(res, statement_.session_.errhp_);
+    }
 }
 
 std::size_t oracle_vector_use_type_backend::size()
@@ -329,7 +349,7 @@ std::size_t oracle_vector_use_type_backend::size()
         // ... and in that case return the actual size
         return actual_size;
     }
-    
+
     if (end_ != NULL && *end_ != 0)
     {
         return *end_ - begin_;
@@ -342,76 +362,21 @@ std::size_t oracle_vector_use_type_backend::size()
 
 std::size_t oracle_vector_use_type_backend::full_size()
 {
-    std::size_t sz = 0; // dummy initialization to please the compiler
-    switch (type_)
-    {
-    // simple cases
-    case x_char:
-        {
-            std::vector<char> *vp = static_cast<std::vector<char> *>(data_);
-            sz = vp->size();
-        }
-        break;
-    case x_short:
-        {
-            std::vector<short> *vp = static_cast<std::vector<short> *>(data_);
-            sz = vp->size();
-        }
-        break;
-    case x_integer:
-        {
-            std::vector<int> *vp = static_cast<std::vector<int> *>(data_);
-            sz = vp->size();
-        }
-        break;
-    case x_long_long:
-        {
-            std::vector<long long> *vp
-                = static_cast<std::vector<long long> *>(data_);
-            sz = vp->size();
-        }
-        break;
-    case x_unsigned_long_long:
-        {
-            std::vector<unsigned long long> *vp
-                = static_cast<std::vector<unsigned long long> *>(data_);
-            sz = vp->size();
-        }
-        break;
-    case x_double:
-        {
-            std::vector<double> *vp
-                = static_cast<std::vector<double> *>(data_);
-            sz = vp->size();
-        }
-        break;
-    case x_stdstring:
-        {
-            std::vector<std::string> *vp
-                = static_cast<std::vector<std::string> *>(data_);
-            sz = vp->size();
-        }
-        break;
-    case x_stdtm:
-        {
-            std::vector<std::tm> *vp
-                = static_cast<std::vector<std::tm> *>(data_);
-            sz = vp->size();
-        }
-        break;
-
-    case x_xmltype:    break; // not supported
-    case x_longstring: break; // not supported
-    case x_statement:  break; // not supported
-    case x_rowid:      break; // not supported
-    case x_blob:       break; // not supported
-    }
-
-    return sz;
+    return get_vector_size(type_, data_);
 }
 
 void oracle_vector_use_type_backend::clean_up()
 {
+    if (type_ == x_longstring || type_ == x_xmltype)
+    {
+        OCILobLocator** const lobps = reinterpret_cast<OCILobLocator**>(buf_);
+        std::size_t const vecSize = size();
+        for (std::size_t i = 0; i != vecSize; ++i)
+        {
+            free_temp_lob(statement_.session_, lobps[i]);
+        }
+    }
+
     if (buf_ != NULL)
     {
         delete [] buf_;
