@@ -1,13 +1,12 @@
 //
-// Copyright (C) 2004-2006 Maciej Sobczak, Stephen Hutton
+// Copyright (C) 2004-2008 Maciej Sobczak, Stephen Hutton
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
 // http://www.boost.org/LICENSE_1_0.txt)
 //
 
 #define SOCI_POSTGRESQL_SOURCE
-#include "soci-postgresql.h"
-#include <soci.h>
+#include "soci/postgresql/soci-postgresql.h"
 #include <libpq/libpq-fs.h> // libpq
 #include <cctype>
 #include <cstdio>
@@ -15,25 +14,21 @@
 #include <ctime>
 #include <sstream>
 
-#ifdef SOCI_PGSQL_NOPARAMS
-#define SOCI_PGSQL_NOBINDBYNAME
-#endif // SOCI_PGSQL_NOPARAMS
-
 #ifdef _MSC_VER
 #pragma warning(disable:4355)
 #endif
 
-using namespace SOCI;
-using namespace SOCI::details;
+using namespace soci;
+using namespace soci::details;
 
 
-PostgreSQLRowIDBackEnd::PostgreSQLRowIDBackEnd(
-    PostgreSQLSessionBackEnd & /* session */)
+postgresql_rowid_backend::postgresql_rowid_backend(
+    postgresql_session_backend & /* session */)
+    : value_(0)
 {
-    // nothing to do here
 }
 
-PostgreSQLRowIDBackEnd::~PostgreSQLRowIDBackEnd()
+postgresql_rowid_backend::~postgresql_rowid_backend()
 {
     // nothing to do here
 }
