@@ -79,7 +79,7 @@ void postgresql_vector_into_type_backend::post_fetch(bool gotData, indicator * i
 
         int const endRow = statement_.currentRow_ + statement_.rowsToConsume_;
 
-        for (int curRow = statement_.currentRow_, i = begin_;
+        for (int curRow = statement_.currentRow_, i = static_cast<int>(begin_);
              curRow != endRow; ++curRow, ++i)
         {
             // first, deal with indicators
@@ -260,7 +260,7 @@ std::size_t postgresql_vector_into_type_backend::size()
 
 std::size_t postgresql_vector_into_type_backend::full_size()
 {
-    std::size_t sz = 0; // dummy initialization to please the compiler
+    std::size_t sz SOCI_DUMMY_INIT(0);
     switch (type_)
     {
         // simple cases
