@@ -5,40 +5,33 @@
 // http://www.boost.org/LICENSE_1_0.txt)
 //
 
-#ifndef SQLITE3_COMMON_H_INCLUDED
-#define SQLITE3_COMMON_H_INCLUDED
+#ifndef SOCI_SQLITE3_COMMON_H_INCLUDED
+#define SOCI_SQLITE3_COMMON_H_INCLUDED
 
+#include "soci/error.h"
+#include <cstddef>
+#include <cstdio>
+#include <cstring>
+#include <ctime>
 #include <vector>
 
-namespace SOCI {
-
-namespace details {
-
-namespace Sqlite3 {
-
-// helper function for parsing datetime values
-void parseStdTm(char const *buf, std::tm &t);
+namespace soci { namespace details { namespace sqlite3 {
 
 // helper for vector operations
 template <typename T>
-std::size_t getVectorSize(void *p)
+std::size_t get_vector_size(void *p)
 {
     std::vector<T> *v = static_cast<std::vector<T> *>(p);
     return v->size();
 }
 
 template <typename T>
-void resizeVector(void *p, std::size_t sz)
+void resize_vector(void *p, std::size_t sz)
 {
     std::vector<T> *v = static_cast<std::vector<T> *>(p);
     v->resize(sz);
 }
 
-} // namespace Sqlite3
-
-} // namespace details
-
-} // namespace SOCI
-
-#endif // SQLITE3_COMMON_H_INCLUDED
+}}} // namespace soci::details::sqlite3
 
+#endif // SOCI_SQLITE3_COMMON_H_INCLUDED
