@@ -19,6 +19,12 @@
 #pragma warning(disable:4355)
 #endif
 
+// This is used instead of tolower() just to avoid warnings about int to char
+// casts inside MSVS std::transform() implementation.
+char toLowerCh(char c) {
+    return static_cast<char>( std::tolower(c) );
+}
+
 using namespace soci;
 using namespace soci::details;
 using namespace sqlite_api;
@@ -32,6 +38,7 @@ sqlite3_statement_backend::sqlite3_statement_backend(
     , databaseReady_(false)
     , boundByName_(false)
     , boundByPos_(false)
+    , hasVectorIntoElements_(false)
     , rowsAffectedBulk_(-1LL)
 {
 }
@@ -267,7 +274,7 @@ sqlite3_statement_backend::bind_and_execute(int number)
                     case dt_string:
                         bindRes = sqlite3_bind_text(stmt_, pos, col.buffer_.constData_, static_cast<int>(col.buffer_.size_), NULL);
                         break;
-                    
+
                     case dt_date:
                         bindRes = sqlite3_bind_text(stmt_, pos, col.buffer_.constData_, static_cast<int>(col.buffer_.size_), SQLITE_TRANSIENT);
                         break;
@@ -337,14 +344,7 @@ sqlite3_statement_backend::execute(int number)
     }
     else
     {
-        if (1 == number)
-        {
-            retVal = load_one();
-        }
-        else
-        {
-            retVal = load_rowset(number);
-        }
+        retVal = fetch(number);
     }
 
     return retVal;
@@ -353,7 +353,7 @@ sqlite3_statement_backend::execute(int number)
 statement_backend::exec_fetch_result
 sqlite3_statement_backend::fetch(int number)
 {
-    if (number > 1)
+    if (hasVectorIntoElements_ || number == 0)
         return load_rowset(number);
     else
         return load_one();
@@ -510,7 +510,7 @@ void sqlite3_statement_backend::describe_column(int colNum, data_type & type,
         dt.resize(siter - dt.begin());
 
     // do all comparisons in lower case
-    std::transform(dt.begin(), dt.end(), dt.begin(), tolower);
+    std::transform(dt.begin(), dt.end(), dt.begin(), toLowerCh);
 
     sqlite3_data_type_map::const_iterator iter = dataTypeMap.find(dt);
     if (iter != dataTypeMap.end())
@@ -561,6 +561,7 @@ sqlite3_standard_use_type_backend * sqlite3_statement_backend::make_use_type_bac
 sqlite3_vector_into_type_backend *
 sqlite3_statement_backend::make_vector_into_type_backend()
 {
+    hasVectorIntoElements_ = true;
     return new sqlite3_vector_into_type_backend(*this);
 }
 
