@@ -47,6 +47,13 @@ void sqlite3_vector_into_type_backend::pre_fetch()
 namespace // anonymous
 {
 
+// MSVS 2015 (only) gives a bogus warning about unreachable code here, suppress
+// it to allow compilation with /WX in the CI builds.
+#ifdef _MSC_VER
+#pragma warning(push)
+#pragma warning(disable:4702) // unreachable code
+#endif
+
 template <typename T>
 void set_in_vector(void* p, int indx, T const& val)
 {
@@ -54,6 +61,26 @@ void set_in_vector(void* p, int indx, T const& val)
     v[indx] = val;
 }
 
+#if defined(_MSC_VER)
+#pragma warning(pop)
+#endif
+
+template <typename T>
+T parse_number_from_string(const char* str)
+{
+    T value;
+    if (!details::cstring_to_integer(value, str))
+        throw soci_error("Cannot convert data");
+
+    return value;
+}
+
+template <>
+double parse_number_from_string(const char* str)
+{
+    return details::cstring_to_double(str);
+}
+
 template <typename T>
 void set_number_in_vector(void *p, int idx, const sqlite3_column &col)
 {
@@ -65,13 +92,10 @@ void set_number_in_vector(void *p, int idx, const sqlite3_column &col)
         case dt_date:
         case dt_string:
         case dt_blob:
-            {
-                T value;
-                if (!details::cstring_to_integer(value, col.buffer_.size_ > 0 ? col.buffer_.constData_ : ""))
-                    throw soci_error("Cannot convert data");
-
-                set_in_vector(p, idx, value);
-            }
+            set_in_vector(p, idx,
+                          parse_number_from_string<T>(col.buffer_.size_ > 0
+                                                        ? col.buffer_.constData_
+                                                        : ""));
             break;
 
         case dt_double:
@@ -319,7 +343,7 @@ std::size_t sqlite3_vector_into_type_backend::size()
     using namespace details;
     using namespace details::sqlite3;
 
-    std::size_t sz = 0; // dummy initialization to please the compiler
+    std::size_t sz SOCI_DUMMY_INIT(0);
     switch (type_)
     {
         // simple cases
