@@ -10,6 +10,7 @@
 #include "soci/oracle/soci-oracle.h"
 #include <iostream>
 #include <string>
+#include <cstdlib>
 #include <cstring>
 #include <ctime>
 
@@ -1067,6 +1068,22 @@ struct longlong_table_creator : table_creator_base
     }
 };
 
+// test using the result of to_number() which uses the default NUMBER type,
+// with precision == scale == 0, with rowset
+TEST_CASE("Oracle to_number with rowset", "[oracle][rowset][to_number]")
+{
+    soci::session sql(backEnd, connectString);
+
+    soci::rowset<soci::row>
+        rs = (sql.prepare << "select to_number('123456789012345') from dual");
+    double d = rs.begin()->get<double>(0);
+    ASSERT_EQUAL_EXACT(d, 123456789012345);
+
+    rs = (sql.prepare << "select to_number(:t) from dual", use(3.14));
+    d = rs.begin()->get<double>(0);
+    ASSERT_EQUAL_EXACT(d, 3.14);
+}
+
 // long long test
 TEST_CASE("Oracle long long", "[oracle][longlong]")
 {
@@ -1538,6 +1555,11 @@ public:
         return true;
     }
 
+    bool treats_empty_strings_as_null() const SOCI_OVERRIDE
+    {
+        return true;
+    }
+
     std::string to_date_time(std::string const &datdt_string) const SOCI_OVERRIDE
     {
         return "to_date('" + datdt_string + "', 'YYYY-MM-DD HH24:MI:SS')";
@@ -1583,6 +1605,12 @@ int main(int argc, char** argv)
         std::exit(1);
     }
 
+    if (!std::getenv("ORACLE_HOME"))
+    {
+        std::cerr << "ORACLE_HOME environment variable must be defined for Oracle tests.\n";
+        std::exit(1);
+    }
+
     test_context tc(backEnd, connectString);
 
     return Catch::Session().run(argc, argv);
